/*
 * I2C support for the X1600.
 *
 * Copyright (C) 2017, 2018, 2021, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/i2c-x1600.h>



// Initialise a channel.

I2c_x1600_channel::I2c_x1600_channel(l4_addr_t start,
                                     enum Clock_identifiers clock,
                                     Cpm_chip *cpm,
                                     uint32_t frequency)
: I2c_channel(start, clock, cpm, frequency)
{
}



// Initialise the I2C controller.

I2c_x1600_chip::I2c_x1600_chip(l4_addr_t start, l4_addr_t end,
                               Cpm_chip *cpm,
                               uint32_t frequency)
: I2c_chip(start, end, cpm, frequency)
{
}



// C language interface functions.

void *x1600_i2c_init(l4_addr_t start, l4_addr_t end, void *cpm, uint32_t frequency)
{
  return (void *) new I2c_x1600_chip(start, end, static_cast<Cpm_chip *>(cpm), frequency);
}

void *x1600_i2c_get_channel(void *i2c, uint8_t channel)
{
  return static_cast<I2c_x1600_chip *>(i2c)->get_channel(channel);
}

uint32_t x1600_i2c_get_frequency(void *i2c_channel)
{
  return static_cast<I2c_x1600_channel *>(i2c_channel)->get_frequency();
}

void x1600_i2c_set_target(void *i2c_channel, uint8_t addr)
{
  static_cast<I2c_x1600_channel *>(i2c_channel)->set_target(addr);
}

void x1600_i2c_start_read(void *i2c_channel, uint8_t buf[], unsigned int total,
                          int stop)
{
  static_cast<I2c_x1600_channel *>(i2c_channel)->start_read(buf, total, stop);
}

void x1600_i2c_read(void *i2c_channel)
{
  static_cast<I2c_x1600_channel *>(i2c_channel)->read();
}

void x1600_i2c_start_write(void *i2c_channel, uint8_t buf[], unsigned int total,
                           int stop)
{
  static_cast<I2c_x1600_channel *>(i2c_channel)->start_write(buf, total, stop);
}

void x1600_i2c_write(void *i2c_channel)
{
  static_cast<I2c_x1600_channel *>(i2c_channel)->write();
}

int x1600_i2c_read_done(void *i2c_channel)
{
  return static_cast<I2c_x1600_channel *>(i2c_channel)->read_done();
}

int x1600_i2c_write_done(void *i2c_channel)
{
  return static_cast<I2c_x1600_channel *>(i2c_channel)->write_done();
}

unsigned int x1600_i2c_have_read(void *i2c_channel)
{
  return static_cast<I2c_x1600_channel *>(i2c_channel)->have_read();
}

unsigned int x1600_i2c_have_written(void *i2c_channel)
{
  return static_cast<I2c_x1600_channel *>(i2c_channel)->have_written();
}

int x1600_i2c_failed(void *i2c_channel)
{
  return static_cast<I2c_x1600_channel *>(i2c_channel)->failed();
}
